/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package tabs.hair;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import utils.Base64;

/**
 * Instances of this class can be used to parse a XML hair style file.
 * The XML file reading can be optimized for reading only some of the file's content.
 * 
 * @see HairStyleXMLHandler#FOR_THUMBNAIL_READING
 * @see HairStyleXMLHandler#FOR_CONTENT_READING
 * @see HairStyleXMLHandler#FOR_HIGHLIGHTS_READING
 * @see HairStyleXMLHandler#FOR_SHADOW_READING
 */
public class HairStyleXMLHandler extends DefaultHandler {

	/**
	 * The hair thumbnail image in {@link Base64}
	 */
	private StringBuilder strThumbnail = null;

	/**
	 * The hair image in {@link Base64}
	 */
	private StringBuilder strContent = null;

	/**
	 * The hair highlights in {@link Base64}
	 */
	private StringBuilder strHighlights = null;

	/**
	 * The hair shadow in {@link Base64}
	 */
	private StringBuilder strShadow = null;

	/**
	 * The hair type
	 */
	private String hairStyle_type;

	/**
	 * The hair title
	 */
	private String hairStyle_title;

	/**
	 * The hair owner's gender
	 */
	private String hairOwner_gender;

	/**
	 * The hair owner's name
	 */
	private String hairOwner_name;

	//flags indicating the current reading part of the XML file
	private boolean thumbnail;
	private boolean content;
	private boolean highlites;
	private boolean shadow;

	//the XML file elements
	private String thumbnailStr = "thumbnail";
	private String contentStr = "content";
	private String highlitesStr = "highlites";
	private String shadowStr = "shadow";

	/**
	 * This instance's current reading mode
	 */
	private int currentReadingMode = -1;

	/**
	 * The XML file reading is optimized for only Thumbnail reading
	 */
	public static final int FOR_THUMBNAIL_READING = 0;

	/**
	 * The XML file reading is optimized for only Content reading
	 */
	public static final int FOR_CONTENT_READING = 1;

	/**
	 * The XML file reading is optimized for only Highlights reading
	 */
	public static final int FOR_HIGHLIGHTS_READING = 2;

	/**
	 * The XML file reading is optimized for only Shadow reading
	 */
	public static final int FOR_SHADOW_READING = 3;

	/**
	 * The latest compatible XML hair file version with this handler
	 */
	private static final Double LATEST_KNOWN_VERSION = 0.1;

	/**
	 * Creates a new instance of {@link HairStyleXMLHandler} that can be
	 * used for reading the complete XML file 
	 */
	public HairStyleXMLHandler() {
		initReadingVars();
	}

	/**
	 * Creates a new instance of {@link HairStyleXMLHandler}
	 * that is optimized for reading from the XML file only
	 * the given readingMode value. 
	 * <p>
	 * The readingMode value is one of the constants defined in
	 * class <code>{@link HairStyleXMLHandler}</code> which is
	 * applicable to instances of this class. The class description
	 * lists the constants that are applicable to the class.
	 * </p>
	 * @param readingMode the type of reading to be done from the XML file
	 */
	public HairStyleXMLHandler(int readingMode) {
		switch (readingMode) {
		case FOR_CONTENT_READING:
			currentReadingMode  = FOR_CONTENT_READING;
			break;
		case FOR_HIGHLIGHTS_READING:
			currentReadingMode  = FOR_HIGHLIGHTS_READING;
			break;
		case FOR_SHADOW_READING:
			currentReadingMode  = FOR_SHADOW_READING;
			break;
		case FOR_THUMBNAIL_READING:
			currentReadingMode  = FOR_THUMBNAIL_READING;
			break;
		default:
			break;
		}
		initReadingVars();
	}

	/**
	 * Reset this instance's variables so that a new parsing can be made
	 */
	public void initReadingVars() {
		strThumbnail = new StringBuilder();
		strContent = new StringBuilder();
		strHighlights = new StringBuilder();
		strShadow = new StringBuilder();
		hairStyle_type = null;
		hairStyle_title = null;
		hairOwner_gender = null;
		hairOwner_name = null;
	}

	/**
	 * The hair thumbnail image in {@link Base64}
	 * @return the hair thumbnail image in {@link Base64}
	 */
	public String getStrThumbnail() {
		return strThumbnail.toString();
	}

	/**
	 * The hair image in {@link Base64}
	 * @return the hair image in {@link Base64}
	 */
	public String getStrContent() {
		return strContent.toString();
	}

	/**
	 * The hair highlights in {@link Base64}
	 * @return the hair highlights in {@link Base64}
	 */
	public String getStrHighlights() {
		return strHighlights.toString();
	}

	/**
	 * The hair shadow in {@link Base64}
	 * @return the hair shadow in {@link Base64}
	 */
	public String getStrShadow() {
		return strShadow.toString();
	}

	/**
	 * The hair owner's gender
	 * @return the hair owner's gender
	 */
	public String getHairOwner_gender() {
		return hairOwner_gender;
	}

	/**
	 * The hair owner's name
	 * @return the hair owner's name
	 */
	public String getHairOwner_name() {
		return hairOwner_name;
	}

	/**
	 * The hair title
	 * @return the hair title
	 */
	public String getHairStyle_title() {
		return hairStyle_title;
	}

	/**
	 * The hair owner's gender
	 * @return the hair owner's gender
	 */
	public String getHairStyle_type() {
		return hairStyle_type;
	}

	@Override
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {

		//reset for the reading contents
		thumbnail = false;
		content = false;
		highlites = false;
		shadow = false;

		//check in which reading mode we are and do the desired actions
		switch (currentReadingMode) {
		case FOR_THUMBNAIL_READING:
			if (qName.equalsIgnoreCase(thumbnailStr)) {
				thumbnail = true;
			}
			break;
		case FOR_CONTENT_READING:
			if (qName.equalsIgnoreCase(contentStr)) {
				content = true;
			}
			break;

		case FOR_HIGHLIGHTS_READING:
			if (qName.equalsIgnoreCase(highlitesStr)) {
				highlites = true;
			}
			break;

		case FOR_SHADOW_READING:
			if (qName.equalsIgnoreCase(shadowStr)) {
				shadow = true;
			}
			break;

		default:
			if (qName.equalsIgnoreCase(thumbnailStr)) {
				thumbnail = true;
			} else if (qName.equalsIgnoreCase(contentStr)) {
				content = true;
			} else if (qName.equalsIgnoreCase(highlitesStr)) {
				highlites = true;
			} else if (qName.equalsIgnoreCase(shadowStr)) {
				shadow = true;
			}
		break;
		}

		if (qName.equalsIgnoreCase("HairStyle")) {
			Double version = Double.parseDouble(attributes.getValue("version"));
			if (version > LATEST_KNOWN_VERSION) {
				//TODO locale ?
				System.err.println("XML file is newer than the suported");
			}
			hairStyle_type = attributes.getValue("type");
			hairStyle_title = attributes.getValue("title");
		}
		if (qName.equalsIgnoreCase("Owner")) {
			hairOwner_gender = attributes.getValue("gender");
			hairOwner_name = attributes.getValue("name");
		}
	}

	@Override
	public void characters(char[] ch, int start, int length) throws SAXException {
		if (thumbnail) {
			strThumbnail.append(ch, start, length);
		} else if (content) {
			strContent.append(ch, start, length);
		} else if (highlites) {
			strHighlights.append(ch, start, length);
		} else if (shadow) {
			strShadow.append(ch, start, length);
		}
	}
}